@@ -31,8 +31,10 @@ pub struct ParserConfig {
 
 #[derive(Debug, Clone, Copy)]
 pub enum Grammar {
-    RuleList,
+    StyleBlock,
     DeclarationList,
+    RuleList,
+    Stylesheet,
     DeclarationValue,
 }
 
